/*
	PrintInfo.h
	Application Kit, Release 2.0
	Copyright (c) 1988, 1989, 1990, NeXT, Inc.  All rights reserved. 
*/

#import <Foundation/NSObject.h>
#import <DPSClient/dpsclient.h>
#import "NSPrintOperation.h"
#import "NSPrinter.h"
#import "NSGraphics.h"

typedef enum _NSPrintingOrientation {
    NSPortraitOrientation		= 0,
    NSLandscapeOrientation		= 1
} NSPrintingOrientation;

typedef enum _NSPrintingPaginationMode {
    NSAutoPagination			= 0,
    NSFitPagination			= 1, // Force image to fit on one page
    NSClipPagination			= 2  // Let image be clipped by page
} NSPrintingPaginationMode;

typedef struct _PrivatePrintInfo *NXPrivatePrintInfo;

@interface NSPrintInfo : NSObject
{
    char               *paperType;
    NSRect              paperRect;
    float             leftPageMargin;
    float             rightPageMargin;
    float             topPageMargin;
    float             bottomPageMargin;
    float               scalingFactor;
    char                pageOrder;
    struct _pInfoFlags {
#ifdef __BIG_ENDIAN__
	unsigned int        orientation:1;
	unsigned int        horizCentered:1;
	unsigned int        vertCentered:1;
	unsigned int        _RESERVEDA:2;
	unsigned int        manualFeed:1;	/* Obsolete */
	unsigned int        allPages:1;
	unsigned int        _RESERVEDC:1;
	unsigned int        horizPagination:2;
	unsigned int        vertPagination:2;
	unsigned int        printerIsOld:1;
	unsigned int        reversePageOrder:1;
	unsigned int        _RESERVEDB:2;
#else
	unsigned int        _RESERVEDB:2;
	unsigned int        reversePageOrder:1;
	unsigned int        printerIsOld:1;
	unsigned int        vertPagination:2;
	unsigned int        horizPagination:2;
	unsigned int        _RESERVEDC:1;
	unsigned int        allPages:1;
	unsigned int        manualFeed:1;       /* Obsolete */
	unsigned int        _RESERVEDA:2;
	unsigned int        vertCentered:1;
	unsigned int        horizCentered:1;
	unsigned int        orientation:1;
#endif
    }                   pInfoFlags;
    int                 firstPage;
    int                 lastPage;
    int                 currentPage;
    int                 copies;
    char               *outputFile;
    DPSContext          context;
    NXPrivatePrintInfo  _privateData;
    char               *printerName;
    char               *printerType;
    char               *printerHost;
    int                 resolution;		/* Obsolete */
    short               pagesPerSheet;
    unsigned short      _reservedPrintInfo1;
    NSPrinter		*printerObject;
    id			jobFeaturesTable;
    const char		*paperFeed;
    unsigned int        _reservedPrintInfo5;
}

+ (NSPrinter *)getDefaultPrinter;
+ setDefaultPrinter:(NSPrinter *)pr;

- setPaperType:(NSString *)type andAdjust:(BOOL)flag;
- (NSString *)paperType;
- setPaperRect:(NSRect)aRect andAdjust:(BOOL)flag;
- (NSRect)paperRect;
- setMarginLeft:(float)leftMargin right:(float)rightMargin top:(float)topMargin bottom:(float)bottomMargin;
- getMarginLeft:(float *)leftMargin right:(float *)rightMargin top:(float *)topMargin bottom:(float *)bottomMargin;
- setScalingFactor:(float)aFloat;
- (float)scalingFactor;
- setOrientation:(NSPrintingOrientation)mode andAdjust:(BOOL)flag;
- (NSPrintingOrientation)orientation;
- setHorizCentered:(BOOL)flag;
- (BOOL)isHorizCentered;
- setVertCentered:(BOOL)flag;
- (BOOL)isVertCentered;
- setHorizPagination:(NSPrintingPaginationMode)mode;
- (NSPrintingPaginationMode)horizPagination;
- setVertPagination:(NSPrintingPaginationMode)mode;
- (NSPrintingPaginationMode)vertPagination;
- setOutputFile:(NSString *)aString;
- (NSString *)outputFile;
- setPageOrder:(NSPrintingPageOrder)mode;
- (NSPrintingPageOrder)pageOrder;
- setAllPages:(BOOL)flag;
- (BOOL)isAllPages;
- setFirstPage:(int)anInt;
- (int)firstPage;
- setLastPage:(int)anInt;
- (int)lastPage;
- (int)currentPage;
- setCopies:(int)anInt;
- (int)copies;
- setContext:(DPSContext)aContext;
- (DPSContext)context;
- setPagesPerSheet:(short)aShort;
- (short)pagesPerSheet;
- setReversePageOrder:(BOOL)flag;
- (BOOL)reversePageOrder;
- setPrinter:(NSPrinter *)pr;
- (NSPrinter *)printer;
- setPaperFeed:(NSString *)str;
- (NSString *)paperFeed;
- initializeJobDefaults;
- setJobFeature:(NSString *)feature toValue:(NSString *)string;
- (NSString *)valueForJobFeature:(NSString *)feature;
- removeJobFeature:(NSString *)feature;
- (const char **)jobFeatures;

@end
